@@ -1,25 +1,29 @@
 /* logjam - a GTK client for LiveJournal.
- * Copyright (C) 2000-2002 Evan Martin <evan@livejournal.com>
+ * Copyright (C) 2000-2003 Evan Martin <evan@livejournal.com>
  *
  * vim: tabstop=4 shiftwidth=4 noexpandtab :
- * $Id: friendgroups.c,v 1.16 2002/10/27 00:11:31 martine Exp $
  */
 
 #include "config.h"
 
-#include <gtk/gtk.h>
+#include "gtk-all.h"
+
+#include <livejournal/editfriendgroups.h>
+
 #include "conf.h"
 #include "util.h"
 #include "network.h"
 #include "friends.h"
 #include "friendgroupedit.h"
+#include "friendgroups.h"
 
 typedef struct {
 	GtkWidget *win;
 	GtkWidget *groups, *inc, *exc;
 	GtkListStore *lgroups, *linc, *lexc;
 
-	GList *friends;
+	JamAccountLJ *account;
+	GSList *friends;
 } FriendGroupsUI;
 
 enum {
@@ -32,21 +36,21 @@ enum {
 };
 
 static int 
-findfreegroup(GtkWidget *win) {
+findfreegroup(FriendGroupsUI *fgui) {
 	int freegroup;
 
 	for (freegroup = 1; freegroup <= 30; freegroup++) {
-		if (conf_nid_by_id(conf_cur_user()->friendgroups, freegroup) == NULL) {
+		if (lj_friendgroup_from_id(jam_account_lj_get_user(fgui->account), freegroup) == NULL) {
 			/* this id isn't in use! */
 			return freegroup;
 		}
 	}
-	lj_warning(win, _("Couldn't find free friend group(?)"));
+	jam_warning(GTK_WINDOW(fgui->win), _("Couldn't find free friend group(?)"));
 	return -1; /* we didn't find one. */
 }
 
 static void
-dlg_add_friend_group(FriendGroupsUI *fgui, FriendGroup *fg) {
+dlg_add_friend_group(FriendGroupsUI *fgui, LJFriendGroup *fg) {
 	GtkTreeIter iter;
 
 	gtk_list_store_append(fgui->lgroups, &iter);
@@ -58,26 +62,28 @@ dlg_add_friend_group(FriendGroupsUI *fgui, FriendGroup *fg) {
 
 static void
 new_cb(GtkWidget *w, FriendGroupsUI *fgui) {
-	FriendGroup *fg;
+	LJUser *u;
+	LJFriendGroup *fg;
 	int freegroup;
 
-	freegroup = findfreegroup(fgui->win);
+	freegroup = findfreegroup(fgui);
 	if (freegroup == -1) return; /* FIXME */
 
-	fg = friend_group_edit_dlg_run(fgui->win, NULL, freegroup);
+	fg = friend_group_edit_dlg_run(GTK_WINDOW(fgui->win), fgui->account, NULL, freegroup);
 	if (fg == NULL) return; /* they cancelled. */
 
 	/* new friend group! */
-	conf_cur_user()->friendgroups = g_list_append(conf_cur_user()->friendgroups, fg);
+	u = jam_account_lj_get_user(fgui->account);
+	u->friendgroups = g_slist_append(u->friendgroups, fg);
 	dlg_add_friend_group(fgui, fg);
 }
 
-static FriendGroup*
+static LJFriendGroup*
 get_selected_fg(FriendGroupsUI *fgui, GtkTreeIter *piter) {
 	GtkTreeSelection *sel;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	FriendGroup *fg;
+	LJFriendGroup *fg;
 
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(fgui->groups));
 	if (!gtk_tree_selection_get_selected(sel, &model, &iter))
@@ -105,14 +111,14 @@ tree_multi_selected(GtkTreeSelection *sel) {
 
 static void
 edit_cb(GtkWidget *w, FriendGroupsUI *fgui) {
-	FriendGroup *fg;
+	LJFriendGroup *fg;
 	GtkTreeIter iter;
 	
 	fg = get_selected_fg(fgui, &iter);
 	if (fg == NULL)
 		return;
 
-	fg = friend_group_edit_dlg_run(fgui->win, fg, -1);
+	fg = friend_group_edit_dlg_run(GTK_WINDOW(fgui->win), fgui->account, fg, -1);
 	if (fg == NULL) return; /* they cancelled. */
 
 	gtk_list_store_set(GTK_LIST_STORE(fgui->lgroups), &iter,
@@ -122,10 +128,11 @@ edit_cb(GtkWidget *w, FriendGroupsUI *fgui) {
 
 static void
 remove_cb(GtkWidget *w, FriendGroupsUI *fgui) {
-	NetRequest *request;
-	NetResult  *result;
-	FriendGroup *fg;
-	GList *l;
+	NetContext *ctx;
+	LJEditFriendGroups *efg;
+	LJUser *u = jam_account_lj_get_user(fgui->account);
+	LJFriendGroup *fg;
+	GSList *l;
 	guint32 gmask;
 	GtkTreeIter iter;
 	
@@ -134,55 +141,59 @@ remove_cb(GtkWidget *w, FriendGroupsUI *fgui) {
 		return;
 	gmask = 1L << fg->id;
 	
-	request = net_request_new("editfriendgroups");
-
-	net_request_set(request, 
-			g_strdup_printf("efg_delete_%d", fg->id),
-			g_strdup("1"));
+	efg = lj_editfriendgroups_new(u);
+	lj_editfriendgroups_add_delete(efg, fg->id);
 
 	/* unlink all friends from this group */
 	/* (unless we do this, these users will erroneously be included in a
 	 * future group that gets this doomed group's id!) */
 	for (l = fgui->friends; l != NULL; l = l->next) {
-		Friend *f = l->data;
-
+		LJFriend *f = l->data;
 		/* only update users in this group (good for large friends lists) */
-		if (f->groupmask & gmask) {
-			f->groupmask &= ~gmask;
-			net_request_set(request, 
-					g_strdup_printf("editfriend_groupmask_%s", f->username),
-					g_strdup_printf("%d", f->groupmask));
-		}
+		if (f->groupmask & gmask)
+			lj_editfriendgroups_add_groupmask(efg, f->username,
+					f->groupmask & ~gmask);
 	}
 
-	result = net_request_run(fgui->win, _("Removing Friend Group..."), request);
-	net_request_free(request);
-
-	if (net_result_succeeded(result)) {
-		gtk_list_store_remove(fgui->lgroups, &iter);
-		conf_cur_user()->friendgroups = g_list_remove(conf_cur_user()->friendgroups, fg);
-		g_free(fg->name);
-		g_free(fg);
+	ctx = net_ctx_gtk_new(GTK_WINDOW(fgui->win), _("Removing Friend Group"));
+	if (!net_run_verb_ctx((LJVerb*)efg, ctx, NULL)) {
+		lj_editfriendgroups_free(efg);
+		net_ctx_gtk_free(ctx);
+		return;
 	}
-	net_result_free(result);
+
+	/* now that it succeeded, actually update our local friends list. */
+	for (l = fgui->friends; l != NULL; l = l->next) {
+		LJFriend *f = l->data;
+		f->groupmask &= ~gmask;
+	}
+
+	gtk_list_store_remove(fgui->lgroups, &iter);
+	u->friendgroups = g_slist_remove(u->friendgroups, fg);
+	g_free(fg->name);
+	g_free(fg);
+
+	lj_editfriendgroups_free(efg);
+	net_ctx_gtk_free(ctx);
 }
 
 static void
 populate_grouplist(FriendGroupsUI *fgui) {
-	GList *lgroup;
+	LJUser *u = jam_account_lj_get_user(fgui->account);
+	GSList *lgroup;
 
-	for (lgroup = conf_cur_user()->friendgroups; lgroup != NULL; lgroup = lgroup->next) {
-		FriendGroup *fg = lgroup->data;
+	for (lgroup = u->friendgroups; lgroup != NULL; lgroup = lgroup->next) {
+		LJFriendGroup *fg = lgroup->data;
 
 		dlg_add_friend_group(fgui, fg);
 	}
 }
 
 static void
-populate_friendlists(FriendGroupsUI *fgui, FriendGroup *fg) {
+populate_friendlists(FriendGroupsUI *fgui, LJFriendGroup *fg) {
 	GtkTreeIter iter;
 	GtkListStore *list;
-	GList *l;
+	GSList *l;
 	guint32 mask;
 
 	gtk_list_store_clear(fgui->linc);
@@ -193,9 +204,9 @@ populate_friendlists(FriendGroupsUI *fgui, FriendGroup *fg) {
 
 	mask = 1L << fg->id;
 	for (l = fgui->friends; l != NULL; l = l->next) {
-		Friend *f = l->data;
+		LJFriend *f = l->data;
 
-		if (!(f->conn & FRIEND_CONN_MY))
+		if (!(f->conn & LJ_FRIEND_CONN_MY))
 			continue;
 
 		if (f->groupmask & mask) {
@@ -216,7 +227,7 @@ static void
 friendgroup_sel_cb(GtkTreeSelection *sel, FriendGroupsUI *fgui) {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
-	FriendGroup *fg = NULL;
+	LJFriendGroup *fg = NULL;
 
 	if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
 		gtk_tree_model_get(model, &iter,
@@ -254,7 +265,7 @@ fg_list_create(FriendGroupsUI *fgui) {
 }
 
 typedef struct {
-	NetRequest *netreq;
+	LJEditFriendGroups *efg;
 	guint32 gmask;
 	gboolean add;
 } FGAddRem;
@@ -262,7 +273,7 @@ typedef struct {
 static void
 friend_addrem_pre_cb(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {
 	FGAddRem *p = data;
-	Friend *f;
+	LJFriend *f;
 	guint32 mask;
 
 	gtk_tree_model_get(model, iter,
@@ -275,15 +286,13 @@ friend_addrem_pre_cb(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
 	else
 		mask &= ~p->gmask;
 
-	net_request_set(p->netreq, 
-			g_strdup_printf("editfriend_groupmask_%s", f->username),
-			g_strdup_printf("%d", mask));
+	lj_editfriendgroups_add_groupmask(p->efg, f->username, mask);
 }
 
 static void
 friend_addrem_post_cb(GtkTreeModel *source, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {
 	FGAddRem *p = data;
-	Friend *f;
+	LJFriend *f;
 
 	gtk_tree_model_get(source, iter,
 			FRIEND_COL_FRIEND, &f,
@@ -298,9 +307,9 @@ friend_addrem_post_cb(GtkTreeModel *source, GtkTreePath *path, GtkTreeIter *iter
 static void
 addrem_cb(FriendGroupsUI *fgui, 
 		GtkTreeView *source, GtkTreeView *dest, gboolean add) {
-	FriendGroup *fg;
-	NetRequest *request;
-	NetResult *result;
+	NetContext *ctx;
+	LJEditFriendGroups *efg;
+	LJFriendGroup *fg;
 	guint32 gmask;
 	FGAddRem rreq;
 	GtkTreeSelection *sel;
@@ -314,25 +323,28 @@ addrem_cb(FriendGroupsUI *fgui,
 
 	gmask = (1L << fg->id);
 
-	request = net_request_new("editfriendgroups");
+	efg = lj_editfriendgroups_new(jam_account_lj_get_user(fgui->account));
 
-	rreq.netreq = request;
+	rreq.efg = efg;
 	rreq.gmask = gmask;
 	rreq.add = add;
 
 	/* this foreach puts all of the new masks into the network request. */
 	gtk_tree_selection_selected_foreach(sel, friend_addrem_pre_cb, &rreq);
 
-	result = net_request_run(fgui->win, "Modifying Friends...", request);
-	net_request_free(request);
-
-	if (net_result_succeeded(result)) {
-		/* and this foreach actually puts all of the new masks
-		 * into our local copy of the friends data. */
-		gtk_tree_selection_selected_foreach(sel, friend_addrem_post_cb, &rreq);
-		populate_friendlists(fgui, fg);
+	ctx = net_ctx_gtk_new(GTK_WINDOW(fgui->win), _("Modifying Friends"));
+	if (!net_run_verb_ctx((LJVerb*)efg, ctx, NULL)) {
+		lj_editfriendgroups_free(efg);
+		net_ctx_gtk_free(ctx);
+		return;
 	}
-	net_result_free(result);
+
+	/* and this foreach actually puts all of the new masks
+	 * into our local copy of the friends data. */
+	gtk_tree_selection_selected_foreach(sel, friend_addrem_post_cb, &rreq);
+	populate_friendlists(fgui, fg);
+	lj_editfriendgroups_free(efg);
+	net_ctx_gtk_free(ctx);
 }
 
 static void
@@ -394,10 +406,10 @@ fg_list_buttonbox(FriendGroupsUI *fgui) {
 	g_signal_connect(G_OBJECT(sel), "changed",
 			G_CALLBACK(rowsel_sensitive_cb), brem);
 
-	box = lj_dialog_buttonbox_new();
-	lj_dialog_buttonbox_add(box, bnew);
-	lj_dialog_buttonbox_add(box, bedit);
-	lj_dialog_buttonbox_add(box, brem);
+	box = jam_dialog_buttonbox_new();
+	jam_dialog_buttonbox_add(box, bnew);
+	jam_dialog_buttonbox_add(box, bedit);
+	jam_dialog_buttonbox_add(box, brem);
 	return box;
 }
 
@@ -438,7 +450,7 @@ static GtkWidget*
 make_include_side(FriendGroupsUI *fgui) {
 	GtkWidget *button;
 
-	fg_make_friendlist(&fgui->inc, &fgui->linc, "Included");
+	fg_make_friendlist(&fgui->inc, &fgui->linc, _("Included"));
 	
 	button = gtk_button_new();
 	gtk_container_add(GTK_CONTAINER(button),
@@ -466,7 +478,7 @@ static GtkWidget*
 make_exclude_side(FriendGroupsUI *fgui) {
 	GtkWidget *button;
 
-	fg_make_friendlist(&fgui->exc, &fgui->lexc, "Excluded");
+	fg_make_friendlist(&fgui->exc, &fgui->lexc, _("Excluded"));
 
 	button = gtk_button_new();
 	gtk_container_add(GTK_CONTAINER(button),
@@ -491,21 +503,26 @@ make_exclude_side(FriendGroupsUI *fgui) {
 }
 
 void
-friendgroups_dialog_new(GtkWidget *parent, GList *friends) {
+friendgroups_dialog_new(GtkWindow *parent, JamAccountLJ *acc, GSList *friends) {
 	FriendGroupsUI *fgui;
 	GtkWidget *paned;
 	GtkWidget *vbox, *hbox;
-	gint dlg_height = 400; /* default first-time size */
 
-	lj_warning(parent, _("This is an experimental feature.  Use friend groups at your own peril, but please tell us about any bugs."));
 	fgui = g_new0(FriendGroupsUI, 1);
+	fgui->account = acc;
 	fgui->friends = friends;
-	fgui->win = lj_dialog_new(parent, _("Friend Groups"), 300, dlg_height);
-	geometry_tie(fgui->win, GEOM_FRIENDGROUPS);
+	fgui->win = gtk_dialog_new_with_buttons(_("Friend Groups"),
+			parent, GTK_DIALOG_MODAL,
+			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+			NULL);
+	jam_win_set_size(GTK_WINDOW(fgui->win), 300, 400);
+	g_signal_connect_swapped(G_OBJECT(fgui->win), "response",
+			G_CALLBACK(gtk_widget_destroy), fgui->win);
 	g_signal_connect_swapped(G_OBJECT(fgui->win), "destroy",
 			G_CALLBACK(g_free), fgui);
 
 	paned = gtk_vpaned_new();
+	geometry_tie_full(GEOM_FRIENDGROUPS, GTK_WINDOW(fgui->win), GTK_PANED(paned));
 
 	vbox = gtk_vbox_new(FALSE, 5);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
@@ -531,13 +548,8 @@ friendgroups_dialog_new(GtkWidget *parent, GList *friends) {
 
 	gtk_paned_pack2(GTK_PANED(paned), hbox, TRUE, TRUE);
 
-	/* try to set the pane position reasonably */
-	gtk_window_get_size(GTK_WINDOW(fgui->win), NULL, &dlg_height);
-	gtk_paned_set_position(GTK_PANED(paned), (int)(dlg_height * 0.382)); /* 1 - 1/phi */
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(fgui->win)->vbox), paned, TRUE, TRUE, 0);
 	gtk_widget_show_all(GTK_DIALOG(fgui->win)->vbox);
 
-	lj_dialog_add_close(fgui->win);
-
 	gtk_widget_show(fgui->win);
 }
